/*
 * SpringProx or K531 SDK :: direct/hal_win32.c
 * --------------------------------------------
 *
 * Linux specific part
 */

/*

  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
  
*/

#include "ref_direct.h"
#ifdef __linux

#include <termios.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>

int CommHandle = -1;

int SerialOpen(char *DeviceName)
{
  struct termios newtio;

  SerialClose();

  /* Open the communication device */
  CommHandle = open(DeviceName, O_RDWR | O_NOCTTY);
  if (CommHandle < 0)
    return E_COMM_ERROR; /* Communication error */

  tcflush(CommHandle, TCIFLUSH);
  
  /* Setup communication : 38400bps, 8 data, 1 stop, no parity, no flow control */  
  memset(&newtio, 0, sizeof(newtio));
  /* CS8  = 8n1 (8bit,no parity,1 stopbit       */
  /* CLOCAL= local connection, no modem control */
  /* CREAD  = enable receiving characters       */
  newtio.c_cflag  = CS8 | CLOCAL | CREAD;
  newtio.c_cflag |= B38400;
  newtio.c_iflag = IGNPAR | IGNBRK;
  newtio.c_oflag = 0;

  /* set input mode (non-canonical, no echo,...) */
  newtio.c_lflag = 0;

  newtio.c_cc[VTIME] = 0; /* inter-character timer unused         */
  newtio.c_cc[VMIN]  = 1; /* blocking read until 1 chars received */

  if (tcsetattr(CommHandle  , TCSANOW, &newtio))
  {
    SerialClose();
    return E_COMM_ERROR;
  }

  /* Success */
  return 0;
}

void SerialClose(void)
{
  if (CommHandle >= 0)
  {
    close(CommHandle);
    CommHandle = -1;
  }
}

int SerialSend(unsigned char *Buffer, int Size)
{
  int done = 0;
  int todo;
  int offset;
  
  todo = Size;
  offset = 0;
  while (todo)
  {
    done = write(CommHandle, &Buffer[offset], todo);
    if (done <= 0)
      return E_COMM_ERROR; /* Communication error */
    todo   -= done;
    offset += done;
  }

  return offset;
}

int SerialRecv(unsigned char *Buffer, int Size)
{
  fd_set fdset;
  struct timeval timeout;
  int    res;
  int done;
  int todo;
  int offset;

  todo = Size;
  offset = 0;
  while (todo)
  {
    /* Init timeout */
    FD_ZERO(&fdset);
    FD_SET(CommHandle, &fdset);

    timeout.tv_sec  = 0;
    if (offset == 0)
      timeout.tv_usec = 500 * 1000; /* Initial timeout */
    else
      timeout.tv_usec =  50 * 1000; /* Afterwards timeout */

    /* Wait */
    res = select(CommHandle + 1, &fdset, NULL, NULL, &timeout);
    if (res > 0)
    {
      /* Data available */
      done = read(CommHandle, &Buffer[offset], todo);
      if (done <= 0)
        return E_COMM_ERROR;
      todo   -= done;
      offset += done;
    } else
    if (res == 0)
    {
      /* Timeout */
      break;
    } else
    {
      /* Select error */
      return E_COMM_ERROR;
    }
  }
   
  return offset;
}

#endif

