#ifndef __REF_DIRECT_H__
#define __REF_DIRECT_H__

/* From protocol.c */
/* --------------- */


/* From polling.c */
/* -------------- */

int PollingProc(void);

int SelectTypeA(unsigned char atq[2], unsigned char uid[12], unsigned char sak[1]);
int SelectTypeB(unsigned char atq[11]);
int Halt(void);

/* From mifare.c */
/* ------------- */
int MifareProc(void);

int MifareReadSector(unsigned char sector, unsigned char data[48], unsigned char key[6]);
int MifareReadBlock(unsigned char block, unsigned char data[16], unsigned char key[6]);

int MifareWriteSector(unsigned char sector, unsigned char data[48], unsigned char key[6]);
int MifareWriteBlock(unsigned char block, unsigned char data[16], unsigned char key[6]);

/* From mif_ul.c */
/* ------------- */

int MifareUlProc(void);

int MifareUlRead4Pages(unsigned char addr, unsigned char data[16]);
int MifareUlWritePage(unsigned char addr, unsigned char data[4]);

/* From tcl.c */
/* ---------- */

int TclProc(void);

int TclConnectTypeA(void);
int TclConnectTypeB(void);
int TclDisconnect(void);
int TclTransmit(const unsigned char iso_in[], unsigned char iso_in_len, unsigned char iso_out[], unsigned char *iso_out_len);

/* From stsr176.c */
/* -------------- */

int StSr176Proc(void);

int StSr176Select(unsigned char chip_id[1], unsigned char uid[8]);
int StSr176ReadBlock(unsigned char addr, unsigned char data[2]);

/* From askcts.c */
/* ------------- */

int AskCtsProc(void);

int AskCtsSelect(unsigned char type[2], unsigned char uid[4]);

int AskCts256ReadBlock(unsigned char addr, unsigned char data[2]);
int AskCts256ResetBlock(unsigned char addr);
int AskCts256SetBlock(unsigned char addr, unsigned char mask[2]);
int AskCts512SelectAll(void);
int AskCts512ReadBlock(unsigned char addr, unsigned char data[2]);
int AskCts512ReadBlocks(unsigned char addr, unsigned char data[8]);
int AskCts512SetBlock(unsigned char addr, unsigned char mask[2]);
int AskCts512WriteBlock(unsigned char addr, unsigned char data[2]);

/* From ascii.c */
/* ------------ */
int AsciiSend(unsigned char *payload, int payload_sz);
int AsciiRecv(unsigned char *payload, int max_payload_sz);

/* From binary.c */
/* ------------- */
int BinarySend(unsigned char *payload, int payload_sz);
int BinaryRecv(unsigned char *payload, int max_payload_sz);

/* Serial stuff, to be provided by OS specific module */
/* -------------------------------------------------- */
int SerialOpen(char *DeviceName);
void SerialClose(void);
int SerialSend(unsigned char *buffer, int length);
int SerialRecv(unsigned char *buffer, int max_length);


/* From ref_direct.c */
/* ----------------- */
int Send(unsigned char *payload, int payload_sz);
int Recv(unsigned char *payload, int max_payload_sz);


/* See ErrorMessage function in ref_direct.c for a text explanation of the codes */
/* ---------------------------------------------------------------------------- */

const char *ErrorMessage(int err_code);

/* Error codes */
#define E_COMM_ERROR      -1
#define E_READER_MUTE     -2
#define E_READER_STOP     -3
#define E_FRAME_BAD_CRC   -4
#define E_FRAME_BAD_LEN   -5
#define E_READER_FATAL    -6
#define E_OVERFLOW        -7
#define E_INTERNAL        -8

/* Status codes */
#define S_NO_CARD          1
#define S_MIFARE_AUTH      2
#define S_CARD_ERROR       3
#define S_BAD_RESP_CODE    4
#define S_BAD_RESP_SIZE    5

/* Trace and debug stuff */
/* --------------------- */

/* Undef this to remove debugging functions */
#define WITH_DEBUG_CODE

/* Trace dialog with the reader */
extern int TraceReader;

/* Trace dialog with the card */
extern int TraceCard;

#endif
