#ifndef __P2P_COMMON_H__
#define __P2P_COMMON_H__

#define LLCP_PTYPE_SYMM              0x00
#define LLCP_PTYPE_AGF               0x02
#define LLCP_PTYPE_CONNECT           0x04
#define LLCP_PTYPE_DISC              0x05
#define LLCP_PTYPE_CC                0x06
#define LLCP_PTYPE_DM                0x07
#define LLCP_PTYPE_FRMR              0x08
#define LLCP_PTYPE_I                 0x0C
#define LLCP_PTYPE_RR                0x0D
#define LLCP_PTYPE_RNR               0x0E

#define LLCP_ADDR_SNEP               0x04
#define LLCP_ADDR_CLIENT             0x20

#define SNEP_REQ_CONTINUE            0x00
#define SNEP_REQ_GET                 0x01
#define SNEP_REQ_PUT                 0x02
#define SNEP_REQ_REJECT              0x7F

#define SNEP_RSP_CONTINUE            0x80
#define SNEP_RSP_SUCCESS             0x81
#define SNEP_RSP_NOT_FOUND           0xC0
#define SNEP_RSP_EXCESS_DATA         0xC1
#define SNEP_RSP_BAD_REQUEST         0xC2
#define SNEP_RSP_NOT_IMPLEMENTED     0xE0
#define SNEP_RSP_UNSUPPORTED_VERSION 0xE1
#define SNEP_RSP_REJECT              0xFF

#define SNEP_VERSION                 0x10

#define LLCP_MAX_PAYLOAD 248

extern const BYTE NFC_ATR_MAGIC_NUMBER[3];
extern const BYTE NFC_ATR_GI_PARAMS[10];


typedef struct
{
  BYTE  dsap;
  BYTE  ssap;
  BYTE  ptype;
  BYTE  payload[LLCP_MAX_PAYLOAD];
  DWORD payload_length;
} LLCP_FRAME_ST;


/* External functions */
/* ------------------ */

BOOL LLCP_SenderEnqueue(LLCP_FRAME_ST *frame);
BOOL LLCP_SenderDequeue(LLCP_FRAME_ST *frame);
BOOL LLCP_Exchange(const LLCP_FRAME_ST *my_frame, LLCP_FRAME_ST *his_frame);
BOOL LLCP_Handle(const LLCP_FRAME_ST *his_frame);

/* Function different for either Initiator or Target mode */
/* ------------------------------------------------------ */

BOOL LLCP_ExchangeEx(const BYTE send_buffer[], DWORD send_len, BYTE recv_buffer[], DWORD *recv_len);

BOOL LLCP_Activate(void);

/* Which execution mode */
#define SERVER              1
#define CLIENT              2
#define SERVER_THEN_CLIENT  3

extern int snep_feature;

#endif
