/*
  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.

  Copyright (c) 2005-2009 PRO ACTIVE SAS - www.proactive-group.com
   
  Ref_Calypso.c
  -------------

  This is the reference applications that shows how to retrieve Calypso
  card's identifier.

  It performs also ISO 14443-A and ISO 14443-B polling, so that we can
  study the side-effects of switching between different configs.
  Clearly we see that going to ISO 14443-A resets the Calypso card.
  
  JDA 14/11/2005 : initial release

*/
#include "products/springprox/springprox.h"
#include "cardware/calypso/calypso_api.h"

#ifdef WIN32
#include <conio.h>              /* For kbhit and getch */
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

static const BYTE AID[] = { '1', 'T', 'I', 'C', '.', 'I', 'C', 'A' };

void exit_handler(void);



int main(int argc, char **argv)
{
  int i;

  char s_buffer[64];

  BYTE uid[12];  /* Serial number of the found card */
  BYTE uidlen;

  BYTE info[64]; /* Protocol-related information of the found card (mandatory for Innovatron protocol) */
  BYTE infolen;

  BYTE chal[8];
  BYTE resp[128];
  CALYPSO_SZ resp_size;

  BYTE data[64];
  CALYPSO_SZ data_size;

  P_CALYPSO_CTX ctx = NULL;

  BYTE rev;
  
  
  WORD protos = PROTO_14443_A|PROTO_14443_B|PROTO_14443_Bi;
  WORD proto;
  DWORD rc = MI_OK;

  char *device = NULL;

  atexit(exit_handler);

  printf("SpringProx (Legacy) SDK\n");
  printf("\n");
  printf("Calypso test software for AFIMB\n");
  printf("-------------------------------\n");
  printf("www.springcard.com\n\n");

  for (i=1; i<argc; i++)
  {
    if (!strcmp("-d", argv[i]))
    {
      if (++i<argc)
      {
        device = argv[i];
        printf("Selecting device '%s'\n", device);
      } else
      {
        printf("Syntax error\n");
        exit(EXIT_FAILURE);
      }
    } else
    if (!strcmp("-A", argv[i]))
    {
    	protos = PROTO_14443_A;
    } else
    if (!strcmp("-B", argv[i]))
    {
    	protos = PROTO_14443_B;
    } else
    if (!strcmp("-I", argv[i]))
    {
    	protos = PROTO_14443_Bi;
    } else
    if (!strcmp("-v", argv[i]))
    {
      CalypsoSetTraceLevel(255);
      CalypsoSetTraceFile("stdout");
    }
  }

  /* Check that the library is working */
  rc = SPROX_GetLibrary(s_buffer, sizeof(s_buffer));
  if (rc != MI_OK)
  {
    printf("Failed to get SpringProx API version\n");
    exit(EXIT_FAILURE);
  }
  printf("SpringProx API is '%s'\n", s_buffer);

  /* Open the SpringProx reader */
  rc = SPROX_ReaderOpen(device);
  if (rc != MI_OK)
  {
    printf("Reader not found\n");
    exit(EXIT_FAILURE);
  }

  rc = SPROX_ReaderGetDevice(s_buffer, sizeof(s_buffer));
  if (rc == MI_OK)
    printf("SpringProx reader found on '%s'\n", s_buffer);

  rc = SPROX_ReaderGetFirmware(s_buffer, sizeof(s_buffer));
  if (rc == MI_OK)
    printf("SpringProx reader is '%s'\n", s_buffer);

  printf("\n");
  
  /* Main loop */
  /* --------- */

#ifdef WIN32
  printf("Waiting for a contactless smartcard... (Press any key to exit)\n\n");
#else
  printf("Waiting for a contactless smartcard... (Press <Ctrl>+C to exit)\n\n");
#endif

  for (;;)
  {
    /* Cleanup the context */
    if (ctx != NULL)
    {
      CalypsoDestroyContext(ctx);
      ctx = NULL;
    }

    /* Wait until a smartcard arrives */
#ifdef WIN32
    while (!kbhit())
#else
    for (;;)
#endif
    {
      uidlen  = sizeof(uid);
      infolen = sizeof(info);
      rc = SPROX_FindEx(protos, &proto, uid, &uidlen, info, &infolen);
      if (rc != MI_NOTAGERR)
        break;
    }

    if (rc == MI_NOTAGERR)
    {
      printf("Cancelled...\n");
      exit(EXIT_FAILURE);
    }

    if (rc != MI_OK) 
    {
      printf("Find failed, error %d\n", rc);
      continue;
    }

    printf("Card found, proto=%04X, UID=", proto);
    for (i=0; i<uidlen; i++)
      printf("%02X", uid[i]);
    printf("\n");

    /* Allocate a context for the library */
    ctx = CalypsoCreateContext();
    if (ctx == NULL)
    {
      printf("Out of memory\n");
      exit(EXIT_FAILURE);
    }

    /* Try to bind our CALYPSO library to the card */
    rc = CalypsoCardBindLegacyEx(ctx, proto, info, infolen);
    if (rc)
    {
      printf("CalypsoCardBindLegacy failed, error %04X (%d).\n", rc, rc);
      continue;
    }

    CalypsoBench(TRUE);


    /* Check this card is actually CALYPSO (ATR parsing or SELECT APPLICATION + FCI parsing) */
    rc = CalypsoCardActivate(ctx, AID, sizeof(AID));
    if (rc)
    {
      if (rc == CALYPSO_CARD_NOT_SUPPORTED)
      {
        printf("This is not a CALYPSO card !\n");
      } else
      {
        printf("CalypsoCardActivate failed, error %04X. (%d)\n", rc, rc);
      }
      continue;
    }
    
    /* Recognize the PO revision */
    /* ------------------------- */
   
    rev = (BYTE) CalypsoCardRevision(ctx);
    printf("Calypso rev. %d\n", rev);
       
    /* Open Secure Session + Read record #1 of SFI #7 (Environment) */
    /* ------------------------------------------------------------ */

    resp_size = sizeof(resp);
    data_size = sizeof(data);
    switch (rev)
    {
      case 1  :
        rc = CalypsoCardOpenSecureSession1(ctx, resp, &resp_size, CALYPSO_KEY_DEBIT, CALYPSO_SFI_ENVIRONMENT, 1, chal, chal, NULL, data, &data_size);
        break;

      case 2  :
        rc = CalypsoCardOpenSecureSession2(ctx, resp, &resp_size, CALYPSO_KEY_DEBIT, CALYPSO_SFI_ENVIRONMENT, 1, chal, chal, NULL, data, &data_size, NULL);
        break;

      case 3  : 
      default :
        rc = CalypsoCardOpenSecureSession3(ctx, resp, &resp_size, CALYPSO_KEY_DEBIT, CALYPSO_SFI_ENVIRONMENT, 1, chal, chal, NULL, data, &data_size, NULL, NULL);
        break;
    }

    if (rc)
    {
      printf("CalypsoCardOpenSecureSession + ReadRecord 7.1 failed, error %04X. (%d)\n", rc, rc);
      goto next;
    }
    
    /* Read Record #1 of SFI #8 (Event Log) */
    /* ------------------------------------ */
    
    data_size = sizeof(data);
    rc = CalypsoCardReadRecord(ctx, CALYPSO_SFI_TRANSPORT_LOG, 1, 0, data, &data_size);

    if (rc)
    {
      printf("CalypsoCardReadRecord 8.1 failed, error %04X (%d).\n", rc, rc);
      goto next;
    }

    /* Read Record #1 of SFI #1E (Contract List) */
    /* ----------------------------------------- */
    
    data_size = sizeof(data);
    rc = CalypsoCardReadRecord(ctx, CALYPSO_SFI_CONTRACT_LIST, 1, 0, data, &data_size);

    if (rc)
    {
      printf("CalypsoCardReadRecord 1E.1 failed, error %04X (%d).\n", rc, rc);
      goto next;
    }

    /* Read Record #1 of SFI #9 (Contracts) */
    /* ------------------------------------ */
    
    data_size = sizeof(data);
    rc = CalypsoCardReadRecord(ctx, CALYPSO_SFI_CONTRACTS, 1, 0, data, &data_size);

    if (rc)
    {
      printf("CalypsoCardReadRecord 9.1 failed, error %04X (%d).\n", rc, rc);
      goto next;
    }
    
    /* Close secure session */
    /* -------------------- */
    
    resp_size = sizeof(resp);
    rc = CalypsoCardCloseSecureSession(ctx, TRUE, chal, resp, &resp_size);

    if (rc == CALYPSO_CARD_DENIED_SAM_SIGN)
    {
      /* SAM's signature is invalid - this is normal, we don't have a SAM! */
      printf("-- Dummy session terminated as expected\n");
    } else
    if (rc)
    {
      printf("CalypsoCardCloseSecureSession failed, error %04X (%d).\n", rc, rc);
      goto next;
    }
    
    /* Send a ratification APDU */
    /* ------------------------ */
    
    rc = CalypsoCardSendRatificationFrame(ctx);

    if (rc)
    {
      printf("CalypsoCardSendRatificationFrame failed, error %04X (%d).\n", rc, rc);
      goto next;
    }
    
    /* Done */
    /* ---- */

    printf("CALYPSO - test OK in %ldms\n", CalypsoBench(TRUE));

    /* Wait for card removal */
next:
    do
    {
      rc = CalypsoCardSendRatificationFrame(ctx);
    } while (!rc);
    
    printf("Card removed : %04X (%d)...\n", rc, rc);

    /* Drop card connection */
    CalypsoCardDispose(ctx);
  }  
}



void exit_handler(void)
{
  SPROX_ReaderClose();
}
