#include "products/springprox/springprox.h"

#ifdef WIN32
#include <conio.h>              /* For kbhit and getch */
#endif
#ifdef __linux
#include <unistd.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>


static const BYTE NFC_CMD_ATR_REQ[2] = { 0xD4, 0x00 };
static const BYTE NFC_CMD_ATR_RES[2] = { 0xD5, 0x01 };
static const BYTE NFC_CMD_WUP_REQ[2] = { 0xD4, 0x02 };
static const BYTE NFC_CMD_WUP_RES[2] = { 0xD5, 0x03 };
static const BYTE NFC_CMD_PSL_REQ[2] = { 0xD4, 0x04 };
static const BYTE NFC_CMD_PSL_RES[2] = { 0xD5, 0x05 };
static const BYTE NFC_CMD_DEP_REQ[2] = { 0xD4, 0x06 };
static const BYTE NFC_CMD_DEP_RES[2] = { 0xD5, 0x07 };
static const BYTE NFC_CMD_DSL_REQ[2] = { 0xD4, 0x08 };
static const BYTE NFC_CMD_DSL_RES[2] = { 0xD5, 0x09 };
static const BYTE NFC_CMD_RLS_REQ[2] = { 0xD4, 0x0A };
static const BYTE NFC_CMD_RLS_RES[2] = { 0xD5, 0x0B };

static const BYTE NFC_ATR_MAGIC_NUMBER[] = { 0x46, 0x66, 0x6D };
static const BYTE NFC_ATR_GI_PARAMS[]    = { 0x01, 0x01, 0x10, 0x03, 0x02, 0x00, 0x01, 0x04, 0x01, 0x96 };

static const char *COM_ANDROID_NPP = "com.android.npp";
static const char *URN_NFC_SN_SNEP = "urn:nfc:sn:snep";

#define ATR_PPI_HAS_GI    0x02
#define ATR_PPI_LRI_DEF   0x30 /* Compliant with NFC-Forum: LRI = 11b */

#define PARAM_SERVICE_NAME 0x06

#define PTYPE_SYMM         0x00
#define PTYPE_CONNECT      0x04

#define ADDR_SERVICE_DISCOVERY_PROTOCOL 0x01
#define ADDR_LOCAL_UNLISTED             0x20

static BYTE pni;

BOOL LLCP_Activate(BYTE uid[], BYTE uidlen)
{
  BYTE buffer[128];
  WORD slen = 0;
  WORD rlen = sizeof(buffer);
  SWORD rc;

  /* Prepare a ATR_REQ command */
  memcpy(&buffer[slen], NFC_CMD_ATR_REQ, 2);
  slen += 2;

  /* NFCID3i */
  memset(&buffer[slen], 0, 10);
  if (uidlen <= 10)
    memcpy(&buffer[slen], uid, uidlen);
  slen += 10;

  /* DIDi */
  buffer[slen++] = 0x00;

  /* BSi */
  buffer[slen++] = 0x00;

  /* BRi */
  buffer[slen++] = 0x00;

  /* PPi */
  buffer[slen++] = ATR_PPI_LRI_DEF | ATR_PPI_HAS_GI;

  /* General bytes defined by NFC-Forum */
  memcpy(&buffer[slen], NFC_ATR_MAGIC_NUMBER, sizeof(NFC_ATR_MAGIC_NUMBER));
  slen += sizeof(NFC_ATR_MAGIC_NUMBER);

  memcpy(&buffer[slen], NFC_ATR_GI_PARAMS, sizeof(NFC_ATR_GI_PARAMS));
  slen += sizeof(NFC_ATR_GI_PARAMS);


  {
    WORD i;
    printf("Nfc<");
    for (i=0; i<slen; i++)
      printf("%02X", buffer[i]);
    printf("\n");
  }

  /* Actual exchange */  
  rc = SPROX_Nfc_Exchange(buffer, slen, buffer, &rlen);
  if (rc != MI_OK)
  {
    printf("Exchange failed, rc=%d\n", rc);
    return FALSE;
  }

  {
    WORD i;
    printf("Nfc>");
    for (i=0; i<rlen; i++)
      printf("%02X", buffer[i]);
    printf("\n");
  }

  /* Check that we got a decently formated ATR_RES */
  if (rlen < 16)
  {
    printf("ATR_RES is too short\n");
    return FALSE;
  }
  if (memcmp(&buffer[0], NFC_CMD_ATR_RES, 2))
  {
    printf("ATR_RES is invalid\n");
    return FALSE;
  }

  /* Check that we retrieve the NFC-Forum magic number */
  if (memcmp(&buffer[17], NFC_ATR_MAGIC_NUMBER, sizeof(NFC_ATR_MAGIC_NUMBER)))
  {
    printf("ATR_RES doesn't announce compliance to NFC-Forum LLCP\n");
    return FALSE;
  }

  printf("NFC-Forum LLCP layer activated\n");

  pni = 0;
  return TRUE;
}

BOOL LLCP_Exchange(const BYTE send_buffer[], WORD send_len, BYTE recv_buffer[], WORD *recv_len)
{
  BYTE buffer[256];
  WORD slen = 0;
  WORD rlen = sizeof(buffer);
  SWORD rc;
  
  if ((send_buffer == NULL) && (send_len != 0))
    return FALSE;
  
  /* Prepare a DEP_REQ command */
  memcpy(&buffer[slen], NFC_CMD_DEP_REQ, 2);
  slen += 2;

  /* PFB = 0 */
  buffer[slen++] = pni & 0x03;

  /* Send buffer */
  memcpy(&buffer[slen], send_buffer, send_len);
  slen += send_len;
  

  {
    WORD i;
    printf("Nfc<");
    for (i=0; i<slen; i++)
      printf("%02X", buffer[i]);
    printf("\n");
  }

  /* Actual exchange */  
  rc = SPROX_Nfc_Exchange(buffer, slen, buffer, &rlen);
  if (rc != MI_OK)
  {
    printf("Exchange failed, rc=%d\n", rc);
    return FALSE;
  }

  {
    WORD i;
    printf("Nfc>");
    for (i=0; i<rlen; i++)
      printf("%02X", buffer[i]);
    printf("\n");
  }

  /* Check that we got a decently formated DEP_RES */
  if (rlen < 3)
  {
    printf("DEP_RES is too short\n");
    return FALSE;
  }
  if (memcmp(&buffer[0], NFC_CMD_DEP_RES, 2))
  {
    printf("DEP_RES is invalid\n");
    return FALSE;
  }

  
  memcpy(recv_buffer, &buffer[3], rlen-3);
  *recv_len = rlen - 3;

  pni++;
  return TRUE;
}

BOOL LLCP_MakeFrame(BYTE buffer[], WORD buffer_size, WORD *actual_length, BYTE ptype, BYTE dsap, BYTE ssap, const BYTE payload[], WORD payload_length)
{
  WORD w;

  w   = (dsap & 0x3F);
  w <<= 4;
  w  |= (ptype & 0x0F);
  w <<= 6;
  w  |= (ssap & 0x3F);

  printf("DSAP=%02X PTYPE=%02X SSAP=%02X --> %04X\n", dsap, ptype, ssap, w);

  buffer[0] = (BYTE) (w / 0x0100);
  buffer[1] = (BYTE) (w % 0x0100);
  *actual_length = 2;

  if (payload != NULL)
  {
    memcpy(&buffer[2], payload, payload_length);
    *actual_length = *actual_length + payload_length;
  }

  return TRUE;
}

BOOL LLCP_ConnectNPP(void)
{
  BYTE buffer[256];
  WORD slen;
  WORD rlen = sizeof(buffer);
  BYTE params[128];
  WORD params_len = 0;

  params[params_len++] = PARAM_SERVICE_NAME;
  params[params_len++] = strlen(COM_ANDROID_NPP);
  memcpy(&params[params_len], COM_ANDROID_NPP, strlen(COM_ANDROID_NPP));
  params_len += strlen(COM_ANDROID_NPP);

  if (!LLCP_MakeFrame(buffer, sizeof(buffer), &slen, PTYPE_CONNECT, ADDR_SERVICE_DISCOVERY_PROTOCOL, ADDR_LOCAL_UNLISTED+1, params, params_len))
    return FALSE;

  if (!LLCP_Exchange(buffer, slen, buffer, &rlen))
    return FALSE;

  return TRUE;
}

BOOL LLCP_ConnectSNEP(void)
{
  BYTE buffer[256];
  WORD slen;
  WORD rlen = sizeof(buffer);
  BYTE params[128];
  WORD params_len = 0;

  params[params_len++] = PARAM_SERVICE_NAME;
  params[params_len++] = strlen(URN_NFC_SN_SNEP);
  memcpy(&params[params_len], URN_NFC_SN_SNEP, strlen(URN_NFC_SN_SNEP));
  params_len += strlen(URN_NFC_SN_SNEP);

  if (!LLCP_MakeFrame(buffer, sizeof(buffer), &slen, PTYPE_CONNECT, ADDR_SERVICE_DISCOVERY_PROTOCOL, ADDR_LOCAL_UNLISTED+1, params, params_len))
    return FALSE;

  if (!LLCP_Exchange(buffer, slen, buffer, &rlen))
    return FALSE;

  return TRUE;
}

BOOL LLCP_Symm(void)
{
  BYTE buffer[256];
  WORD slen;
  WORD rlen = sizeof(buffer);

  if (!LLCP_MakeFrame(buffer, sizeof(buffer), &slen, PTYPE_SYMM, 0, 0, NULL, 0))
    return FALSE;

  if (!LLCP_Exchange(buffer, slen, buffer, &rlen))
    return FALSE;

  return TRUE;
}

int main(int argc, char **argv)
{
  SWORD rc;
  int i;
  char s_buffer[64];
  BYTE uid[12];
  BYTE uid_len = 12;
  DWORD features;
  WORD proto = 0;

  char *device = NULL;

  if (argc > 1)
    device = argv[1];

  printf("SpringCard SDK for K531/SpringProx-CF/CSB Legacy\n");
  printf("\n");
  printf("PEERTOPEER\n");
  printf("----------\n");
  printf("www.springcard.com\n\n");

  /* Open reader */
  /* ----------- */

  rc = SPROX_GetLibrary(s_buffer, sizeof(s_buffer));
  if (rc != MI_OK)
  {
    printf("Failed to get API version\n");
    goto done;
  }
  printf("API version : %s\n", s_buffer);
  printf("\n");

  /* Open reader */
  /* ----------- */

  rc = SPROX_ReaderOpen(device);
  if (rc != MI_OK)
  {
    printf("Reader not found\n");
    goto done;
  }

  printf("Reader found\n");
  rc = SPROX_ReaderGetDevice(s_buffer, sizeof(s_buffer));
  if (rc == MI_OK)
    printf("Reader found on %s\n", s_buffer);

  rc = SPROX_ReaderGetFirmware(s_buffer, sizeof(s_buffer));
  if (rc == MI_OK)
    printf("Reader firwmare is %s\n", s_buffer);

  rc = SPROX_ReaderGetFeatures(&features);
  if ((rc == MI_OK) && !(features & 0x00000002))
  {
    printf("Warning: this reader is not 15693-enabled\n");
  }

  printf("\n------------------------------\n");


  /* Set RF field to ON */
  SPROX_ControlRF(TRUE);

  printf("Waiting for target... (Press <Ctrl>+C to exit)\n\n");

  for (;;)
  {
    rc = MI_OK;
    /* Not too often... */
#ifdef WIN32
    if (kbhit())
      break;
    Sleep(50);
#endif
#ifdef __linux
    usleep(5000);
#endif

    for (;;)
    {
      uid_len = sizeof(uid);
      rc = SPROX_Find(PROTO_FELICA|PROTO_14443_A, &proto, uid, &uid_len);
	    if (rc == MI_NOTAGERR)
        continue;                  /* No more tag (or no tag at all) */

      if (rc != MI_OK)
      {
        printf("\n[Failed]\n");
		     printf("  %s (%d)\n", SPROX_GetErrorMessage(rc), rc);
        if (rc < -128)
          goto close;           /* Fatal error */
      }

      if (rc == MI_OK)
      {
        printf("\nTag found [OK]\n");
		    printf("Family=");

		    switch (proto)
        {
		      case PROTO_14443_A	: printf("Nfc-A\n"); break;
		      case PROTO_FELICA   : printf("Nfc-F\n"); break;
		      default             : printf("%04X\n", proto);
        }
    
		    printf("NFCID=");
		    for (i=0; i<uid_len; i++)
		      printf("%02X", uid[i]);
		    printf("\n");

		    break;
	    }
	  }

	  if (proto == PROTO_14443_A)
	  {
      BYTE atq[2];
      BYTE snr[12];
      BYTE snrlen = 12;
      BYTE sak[1];
      WORD w_atq;

      rc = SPROX_A_SelectAny(atq, snr, &snrlen, sak);
      if (rc == MI_OK)
      {
		    w_atq = (atq[1] << 8) | atq[0];
		    printf("ATQ of Nfc-A device is %04X\n", w_atq);
        SPROX_A_Halt();
      }
    }

	  if (proto == PROTO_FELICA)
    {
      if (!LLCP_Activate(uid, uid_len))
      {
        printf("LLCP_Activate failed\n");
        continue;
      }

      if (!LLCP_ConnectSNEP())
      {
        printf("LLCP_ConnectSNEP failed\n");
        continue;
      }

      /*
      if (!LLCP_ConnectNPP())
      {
        printf("LLCP_ConnectNPP failed\n");
        continue;
      }
      */

      while (LLCP_Symm());

    }

  }

close:
  /* Close the reader */
  SPROX_ControlRF(FALSE);
  SPROX_ReaderClose();

done:
  /* Display last error */
  if (rc == MI_OK)
  {
    printf("Done\n");
  } else
  {
    printf("%s (%d)\n", SPROX_GetErrorMessage(rc), rc);
  }
  return 0;
}
