/*
  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.

  Copyright (c) 2009-2014 PRO ACTIVE SAS, FRANCE - www.springcard.com
   
  ref_wait.c
  ----------

  This is the reference applications that shows how to ask the reader
  to wake us when a card arrives.
  
  LTX 25/08/2009 : initial release
  JDA 24/01/2012 : minor changes to adapt to release 1.62 of the SDK
  JDA 13/08/2014 : moved to Visual C++ Express 2010, added the 'A'
                   suffix to all text-related functions 

*/
#include "products/springprox/springprox.h"

#ifdef WIN32
#include <conio.h>              /* For kbhit and getch */
#endif
#ifdef __linux
#include <unistd.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

int main(int argc, char **argv)
{
  SWORD rc;
  int i;
  char s_buffer[64];
  BYTE uid[12];
  BYTE uid_len = 12;
  DWORD features;
  WORD proto = 0;

  char *device = NULL;

  if (argc > 1)
    device = argv[1];

  printf("SpringCard SDK for K663/K632/K531/SpringProx-CF/CSB Legacy\n");
  printf("\n");
  printf("ref_wait : multi-protocol card polling in reader sample\n");
  printf("--------\n");
  printf("This is a free and unsupported sample provided by www.springcard.com\n");
  printf("Please read LICENSE.txt for details\n");
  printf("\n");

  /* Open reader */
  /* ----------- */

  rc = SPROX_GetLibraryA(s_buffer, sizeof(s_buffer));
  if (rc != MI_OK)
  {
    printf("Failed to get API version\n");
    goto done;
  }
  printf("API version : %s\n", s_buffer);
  printf("\n");

  /* Open reader */
  /* ----------- */

  rc = SPROX_ReaderOpenA(device);
  if (rc != MI_OK)
  {
    printf("Reader not found\n");
    goto done;
  }

  printf("Reader found\n");
  rc = SPROX_ReaderGetDeviceA(s_buffer, sizeof(s_buffer));
  if (rc == MI_OK)
    printf("Reader found on %s\n", s_buffer);

  rc = SPROX_ReaderGetFirmwareA(s_buffer, sizeof(s_buffer));
  if (rc == MI_OK)
    printf("Reader firwmare is %s\n", s_buffer);

  rc = SPROX_ReaderGetFeatures(&features);
  if ((rc == MI_OK) && !(features & 0x00000002))
  {
    printf("Warning: this reader is not 15693-enabled\n");
  }

  printf("\n------------------------------\n");


  printf("Waiting for tags... (Press <Ctrl>+C to exit)\n\n");

  uid_len = sizeof(uid);

  rc = SPROX_FindWait(PROTO_ANY, &proto, uid, &uid_len, 0xFFFF, 100); 
	if (rc != MI_OK)
	{
		printf("\n[Failed]\n");
  } else
  {
		printf("\nTag found [OK]\n");
		printf("Family=");

		switch (proto)
		{
		  case PROTO_14443_A	      : printf("ISO/IEC 14443-A (or NXP Mifare)\n"); break;
		  case PROTO_14443_B	      : printf("ISO/IEC 14443-B\n"); break;
		  case PROTO_15693            : printf("ISO/IEC 15693\n"); break;
		  case PROTO_ICODE1           : printf("NXP ICODE-1\n"); break;
		  case PROTO_INSIDE_PICO      : printf("Inside Contactless PicoTAG\n"); break;
		  case PROTO_ST_SR		      : printf("S.T. MicroElectronics SR\n"); break;
		  case PROTO_ASK_CTS	      : printf("ASK CTS256B/CTS512B\n"); break;
		  case PROTO_INNOVATRON       : printf("Calypso (Innovatron protocol)\n"); break;
		  case PROTO_INNOVISION_JEWEL : printf("Innovision Jewel\n"); break;
		  case PROTO_KOVIO_BARCODE    : printf("Kovio RF barcode\n"); break;
		  case PROTO_FELICA           : printf("Felica\n"); break;
		  default                     : printf("%04X\n", proto);
		}
    
		printf("UID=");
		for (i=0; i<uid_len; i++)
		  printf("%02X", uid[i]);
		printf("\n");
    
    printf("SUCCESS!\n");    
	}

  /* Close the reader */
  SPROX_ControlRF(FALSE);
  SPROX_ReaderClose();

done:
  /* Display last error */
  if (rc != MI_OK)
    printf("%s (%d)\n", SPROX_GetErrorMessageA(rc), rc);

  return EXIT_SUCCESS;
}
