﻿/*
 * ******************************************************************
 * Springcard - CsbDiag2
 * Scripting application using SpringProx API
 * Copyright (c) 2004-2014 Pro Active SAS, France
 * Go to www.springcard.com for information and updates
 * ******************************************************************
 */
using System;
using System.Drawing;
using System.Windows.Forms;
using NLog;

namespace CsbDiag2
{
	/// <summary>
	/// Description of FeaturesList.
	/// </summary>
	public partial class FeaturesList : Form
	{
		private string Features = "";
		
		public FeaturesList()
		{
			InitializeComponent();
		}
		
		public void SetFeatures(string features)
		{
			this.Features = features;
		}
		
		void BtnOkClick(object sender, EventArgs e)
		{
			Close();
		}
		
		void FeaturesListLoad(object sender, EventArgs e)
		{
			ExplainList.Text = "";
			ExplainList.Text += "Explanation" + Environment.NewLine;
			
			if(this.Features.Equals(""))
			{
				ExplainList.Text += "Warning : this version of the reader firmware doesn't provide the GetFeatures function." + Environment.NewLine;
				ExplainList.Text += "The information displayed here are only default values, and therefore are not reliable." + Environment.NewLine;
			}
			
			Int64 cpb = 0; 
			try {
				cpb = Convert.ToInt64(this.Features, 16);	
			} catch (Exception Ex) {
				
				MessageBox.Show("There was an error while trying to translate the features list to a number : " + Ex.Message);
				Close();
			}			 

			if ((cpb & 0x00000001) != 0)
				ExplainList.Text += "- ISO 14443 (A and B)" + Environment.NewLine;
					
			if ((cpb & 0x00000002) != 0)
				ExplainList.Text += "- ISO 15693" + Environment.NewLine;
			
			if ((cpb & 0x00000020) != 0)
				ExplainList.Text += "- GemCore smartcard reader" + Environment.NewLine;
			
			if ((cpb & 0x00000040) != 0)
				ExplainList.Text += "- SAGEM MSO CBM fingerprints reader" + Environment.NewLine;

			ExplainList.Text += "Supported communication protocols :" + Environment.NewLine;
			ExplainList.Text += "- OSI 3964" + Environment.NewLine;
			
			if ((cpb & 0x00000800) != 0)
				ExplainList.Text += "- Binary/Bus" + Environment.NewLine;
			
			if ((cpb & 0x00000200) != 0)
				ExplainList.Text += "- Binary" + Environment.NewLine;
			
			if ((cpb & 0x00000100) != 0)
				ExplainList.Text += "- ASCII" + Environment.NewLine;

			if ((cpb & 0x00001000) != 0)
				ExplainList.Text += "Device has an RS-485 physical interface" + Environment.NewLine;
			
			if ((cpb & 0x00002000) != 0)
				ExplainList.Text += "Device has separate in/out buffers" + Environment.NewLine;
			
			if ((cpb & 0x00004000) != 0)
				ExplainList.Text += "Device is able to communicate at 115200bps" + Environment.NewLine;
			
			if ((cpb & 0x00008000) != 0)
				ExplainList.Text += "Device has XXL buffers" + Environment.NewLine;

			if ((cpb & 0x000F0000) != 0)
				ExplainList.Text += "USB device capabilities :" + Environment.NewLine;
						
			if ((cpb & 0x00010000) != 0)
				ExplainList.Text += "- CDC-ACM class (\"legacy\" virtual comm port)" + Environment.NewLine;
			
			if ((cpb & 0x00020000) != 0)
				ExplainList.Text += "- CCID class (PC/SC smartcard reader)" + Environment.NewLine;
			
			if ((cpb & 0x00040000) != 0)
				ExplainList.Text += "- HID class (keyboard emulation)" + Environment.NewLine;

			if ((cpb & 0xFFF00000) != 0)
				ExplainList.Text += "Advanced capabilities :" + Environment.NewLine;
			
			if ((cpb & 0x00400000) != 0)
				ExplainList.Text += "- FEED (Flash or E2PROM for persistent data storage)" + Environment.NewLine;
			
			if ((cpb & 0x01000000) != 0)
				ExplainList.Text += "- Console access" + Environment.NewLine;
			
			if ((cpb & 0x02000000) != 0)
				ExplainList.Text += "- Noralsy specific functions" + Environment.NewLine;				
		}
	}
}
