unit uCSBDiag;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, Buttons, ComCtrls, Registry, IniFiles, XPMan,
  ImgList, ShellApi, uUsbNotification, jpeg, Spin;

type
  TfCSBDiag = class(TForm)
    sbStatus: TStatusBar;
    pTop: TPanel;
    bsTop: TBevel;
    lbTitle: TLabel;
    lbSubTitle1: TLabel;
    pBot: TPanel;
    bsBot: TBevel;
    lbCopy: TLabel;
    btnOK: TBitBtn;
    btnClose: TBitBtn;
    pLeft: TPanel;
    pMain: TPanel;
    gbDllRelease: TGroupBox;
    lbDllVersion: TLabel;
    eDllVersion: TEdit;
    gbDevice: TGroupBox;
    gbReader: TGroupBox;
    lbCsbVersion: TLabel;
    lbCsbChipsetHWID: TLabel;
    lbCsbChipsetSNR: TLabel;
    eCsbVersion: TEdit;
    eCsbChipsetHWID: TEdit;
    eCsbChipsetSNR: TEdit;
    lbDllLocation: TLabel;
    eDllLocation: TEdit;
    tmrFirst: TTimer;
    lbCsbFeatures: TLabel;
    eCsbFeatures: TEdit;
    ckAuto: TCheckBox;
    ckUsb: TCheckBox;
    ckSerial: TCheckBox;
    ckOther: TCheckBox;
    eOther: TEdit;
    cbSerial: TComboBoxEx;
    CommImages: TImageList;
    lbDeviceInfo: TLabel;
    btnAdvanced: TBitBtn;
    btnExplain: TBitBtn;
    btnBench: TBitBtn;
    btnTest: TBitBtn;
    lbWeb: TLabel;
    lbVersion: TLabel;
    gbAction: TGroupBox;
    btnPolling: TBitBtn;
    lbCard: TLabel;
    tmrPolling: TTimer;
    imgBackground: TImage;
    imgHeader: TImage;
    imgBranded: TImage;
    btnDesactDebug: TBitBtn;
    btnActDebug: TBitBtn;
    btnChangeLevel: TBitBtn;
    spinLevel: TSpinEdit;
    lbDebugLevel: TLabel;
    eDebugPath: TEdit;
    lbDebugPath: TLabel;
    procedure tmrFirstTimer(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure SaveToReg;
    procedure LoadFromReg;
    procedure LoadCommList;
    procedure ApplyDeviceName;
    procedure btnTestClick(Sender: TObject);
    procedure btnOKClick(Sender: TObject);
    procedure btnCloseClick(Sender: TObject);
    procedure btnExplainClick(Sender: TObject);
    procedure btnBenchClick(Sender: TObject);
    procedure btnAdvancedClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure ckAutoClick(Sender: TObject);
    procedure lbWebClick(Sender: TObject);
    procedure btnPollingClick(Sender: TObject);
    procedure tmrPollingTimer(Sender: TObject);
    procedure ConnectStatus(connected : boolean);
    procedure UsbNotification(Sender : TObject);
    procedure btnDesactDebugClick(Sender: TObject);
    procedure btnActDebugClick(Sender: TObject);
    procedure btnChangeLevelClick(Sender: TObject);

  private
    DeviceName : string;
    UsbNotifier : TUsbNotification;
  public
    { Dclarations publiques }
  end;

var
  fCSBDiag: TfCSBDiag;

implementation

{$R *.dfm}

uses uCsbDyn, uCSBCommMore;

procedure TfCSBDiag.tmrFirstTimer(Sender: TObject);
var
   DllVersion : string;
   DllPath    : string;
   rc         : sword;
begin
   tmrFirst.Enabled := false;
   sbStatus.SimpleText := 'Please wait, looking for SpringProx library...';
   Application.ProcessMessages;
   Screen.Cursor := crHourGlass;

   rc := GetLibrary(DllVersion, DllPath);
   if rc = -1000 then
   begin
      screen.Cursor := crDefault;
      sbStatus.SimpleText := GetErrorMessage(rc);
      Application.MessageBox('springprox.dll is not available in program directory, nor in system default search path. Please copy springprox.dll in appropriate directory.', 'SpringProx API not available', 0);
   end else
   if rc = -1001 then
   begin
      screen.Cursor := crDefault;
      sbStatus.SimpleText := GetErrorMessage(rc);
      Application.MessageBox('springprox.dll has been found, appears to be invalid or corrupted. Please replace springprox.dll with an up-to-date release.', 'SpringProx API not available', 0);
   end else
   begin
      eDllVersion.Text  := DllVersion;
      eDllLocation.Text := DllPath;
      sbStatus.SimpleText := '';
      screen.Cursor := crDefault;
      btnTest.Enabled := true;
      btnOK.Enabled := true;
      btnOK.SetFocus;
   end;
   btnClose.Enabled := true;
end;

procedure TfCSBDiag.tmrPollingTimer(Sender: TObject);
var
   rc : sword;
   uid : string;
   info : string;
begin
   if btnPolling.Tag = 0 then exit;

   rc := Find(uid, info);

   if rc = 0 then
   begin
      lbCard.Caption := info + ' : ' + uid;
   end else
   if rc = -1 then
   begin
      lbCard.Caption := 'No card on the reader';
   end else
   begin
      lbCard.Caption := GetErrorMessage(rc);
   end;
   
   
end;

procedure TfCSBDiag.FormShow(Sender: TObject);
begin
   Application.ProcessMessages;
   LoadFromReg;
   LoadCommList;
   if (DeviceName = 'auto') or (DeviceName = '') then
   begin
      ckAuto.Checked := true;
   end else
   if DeviceName = 'USB' then
   begin
      ckUsb.Checked := true;
   end else
   if copy(DeviceName,1,3) = 'COM' then
   begin
      ckSerial.Checked := true;
   end else
   begin
      ckOther.Checked := true;
      eOther.Text := DeviceName;
   end;
   tmrFirst.Enabled := true;

   UsbNotifier := TUsbNotification.Create(self);
   UsbNotifier.OnUSBArrival := UsbNotification;
   UsbNotifier.OnUSBRemove := UsbNotification;
end;

procedure TfCSBDiag.UsbNotification(Sender: TObject);
var
   s : string;
begin
   s := sbStatus.SimpleText;
   sbStatus.SimpleText := 'Updating list of communication devices...';
   screen.Cursor := crHourglass;
   Application.ProcessMessages;
   LoadCommList;
   screen.Cursor := crDefault;
   sbStatus.SimpleText := s;
end;


procedure TfCSBDiag.SaveToReg;
var
   r : TRegistry;
   f : boolean;
begin
   f := false;
   r := TRegistry.Create;
   try
      r.RootKey := HKEY_LOCAL_MACHINE;
      r.OpenKey('Software\SpringCard', true);
      r.CloseKey;
      if r.OpenKey('Software\SpringCard\SpringProx', true) then
      begin
         if Length(DeviceName) > 0 then
         begin
            r.WriteString('DeviceName', DeviceName);
         end else
         begin
            r.DeleteValue('DeviceName');
         end;
         r.CloseKey;
         f := true;
      end;
   finally
      r.Free;
   end;
   r := TRegistry.Create;
   try
      r.RootKey := HKEY_CURRENT_USER;
      r.OpenKey('Software\SpringCard', true);
      r.CloseKey;
      if r.OpenKey('Software\SpringCard\SpringProx', true) then
      begin
         if (f) or (Length(DeviceName) = 0) then
         begin
            r.DeleteValue('DeviceName');
         end else
         begin
            r.WriteString('DeviceName', DeviceName);
         end;
         r.CloseKey;
      end;
   finally
      r.Free;
   end;
end;

procedure TfCSBDiag.LoadFromReg;
var
   r : TRegistry;
   s : string;
begin
   DeviceName := '';
   r := TRegistry.Create;
   try
      r.RootKey := HKEY_CURRENT_USER;
      if r.OpenKeyReadOnly('Software\SpringCard\SpringProx') then
         DeviceName := r.ReadString('DeviceName');
      r.CloseKey;
   finally
      r.Free;
   end;
   if DeviceName = '' then
   begin
      r := TRegistry.Create;
      try
         r.RootKey := HKEY_LOCAL_MACHINE;
         if r.OpenKeyReadOnly('Software\SpringCard\SpringProx') then
            if DeviceName = '' then DeviceName := r.ReadString('DeviceName');
         r.CloseKey;
      finally
         r.Free;
      end;
   end;

   r := TRegistry.Create;
   try
      r.RootKey := HKEY_LOCAL_MACHINE;
      if r.OpenKey('Software\SpringCard\SpringProx', true) then
      begin
        s := r.ReadString('DebugFile');
        if s = '' then
        begin
          eDebugPath.Text := 'C:\Temp\springprox.log';
        end else
        begin
          eDebugPath.Text := s;
        end;

        s := r.ReadString('DebugLevel');
        if (s = '0') or (s = '') then
        begin
          btnDesactDebug.Visible := FALSE;
          btnActDebug.Visible := TRUE;
          btnChangeLevel.Visible := FALSE;
          spinLevel.Visible := FALSE;
          lbDebugLevel.Visible := FALSE;
        end else
        begin
          btnDesactDebug.Visible := TRUE;
          btnActDebug.Visible := FALSE;
          btnChangeLevel.Visible := TRUE;
          spinLevel.Visible := TRUE;
          lbDebugLevel.Visible := TRUE;

          spinLevel.Text := s;

          r.WriteString('DebugFile', eDebugPath.Text);
          r.CloseKey;
          r.OpenKey('Software\SpringCard\SpringProx', true);
          r.WriteString('DebugFile', eDebugPath.Text);
          r.WriteString('DebugLevel', spinLevel.Text);
        end;
      end else
      begin
        btnDesactDebug.Visible := FALSE;
        btnActDebug.Visible := TRUE;
        btnChangeLevel.Visible := FALSE;
        spinLevel.Visible := FALSE;
        lbDebugLevel.Visible := FALSE;
      end;
      r.CloseKey;
   finally
      r.Free;
   end;
end;

procedure TfCSBDiag.LoadCommList;
var
   CommList : TStringList;
   it       : TComboExItem;
   sl       : TStringList;
   f        : boolean;
   n, v, t  : string;
   i, j     : integer;
   pid, vid : integer;
begin
   CommList := TStringList.Create;
   GetCommList(CommList);
   cbSerial.ItemsEx.Clear;
   for i := 0 to CommList.Count - 1 do
   begin
      n := CommList.Names[i];
      v := CommList.Values[n] + ';;;;;;';

      // Get USB VID & PID
      t := v;
      for j := 1 to 4 do t := copy(t, pos(';', t)+1, length(t));
      t := copy(t, 1, pos(';', t)-1);
      vid := StrToIntDef('$' + t, 0);
      t := v;
      for j := 1 to 5 do t := copy(t, pos(';', t)+1, length(t));
      t := copy(t, 1, pos(';', t)-1);
      pid := StrToIntDef('$' + t, 0);

      t := copy(v, 1, pos(';', v)-1);

      f := false;

      if vid = $0403 then
      begin
         // FTDI
         if (pid = $d968) or (pid = $d969)  or (pid = $d96a) then f := true;  

      end else
      if vid = $1C34 then
      begin
         // Pro Active or SpringCard
         f := true;

         // Limit us to CSB family
         if ((pid and $FF00) <> $7000) then continue;
      end;

//      ShowMessage(t + '|' + vid + '|' + pid);
      sl := TStringList.Create;
      it := cbSerial.ItemsEx.Add;
      it.Caption := n + ' : ' + t;
      if f then it.ImageIndex := 2 else it.ImageIndex := 0;

      sl.Free;
      if DeviceName = n then
      begin
         ckSerial.Checked := true;
         cbSerial.ItemIndex := i;
      end;
   end;
   if (cbSerial.ItemIndex = -1) and (cbSerial.ItemsEx.Count > 0) then cbSerial.ItemIndex := 0;
   CommList.Free;
end;

procedure TfCSBDiag.ConnectStatus(connected : boolean);
begin
   if connected then
   begin
      btnTest.Caption := 'Disconnect';
      btnTest.Tag := 1;
   end else
   begin
      btnTest.Caption := 'Test communication';
      btnTest.Tag := 0;

      eCsbVersion.Text := '';
      eCsbChipsetHwID.Text := '';
      eCsbChipsetSnr.Text := '';
      eCsbFeatures.Text := '';
      lbDeviceInfo.Caption := '';
   end;

   btnExplain.Enabled := connected;
   btnBench.Enabled := connected;
   btnPolling.Enabled := connected;
   btnAdvanced.Enabled := not connected;

   ckAuto.Enabled := not connected;
   ckUsb.Enabled := not connected;
   ckSerial.Enabled := not connected;
   cbSerial.Enabled := (not connected) and ckSerial.Checked;
   ckOther.Enabled := not connected;
   eOther.Enabled := (not connected) and ckOther.Checked;
end;

procedure TfCSBDiag.btnTestClick(Sender: TObject);
var
   rc : sword;
   ReaderDevice : string;
   ReaderFirmware : string;
   ReaderRc500HWID : string;
   ReaderRc500SNR : string;
   ReaderSettings : dword;
   ReaderConsts : dword;
   ReaderFeatures : dword;
begin
   if btnTest.Tag <> 0 then
   begin
      btnPolling.Tag := 0;
      btnPolling.Caption := 'Contactless polling';
      lbCard.Caption := '';
      CloseReader;
      ConnectStatus(false);
      exit;
   end;

   ApplyDeviceName;
   ReaderDevice := DeviceName;

   if (ReaderDevice = '') then ReaderDevice := 'auto';

   if (ReaderDevice = 'auto') then
   begin
      sbStatus.SimpleText := 'Please wait, auto-detection in progress (this may take up to 60 seconds)...';
   end else
   begin
      sbStatus.SimpleText := 'Please wait, looking for reader on ' + DeviceName;
   end;

   btnOK.Enabled := false;
   btnClose.Enabled := false;
   Screen.Cursor := crHourglass;
   Application.ProcessMessages;

   ReaderSettings := $00000000;
   case fCsbCommMore.rgCommProtocol.ItemIndex of
   1 : ReaderSettings := ReaderSettings or $00000004;
   2 : ReaderSettings := ReaderSettings or $00000006;
   3 : ReaderSettings := ReaderSettings or $00000005;
   end;
   case fCsbCommMore.rgCommSpeed.ItemIndex of
   1 : ReaderSettings := ReaderSettings or $00000010;
   2 : ReaderSettings := ReaderSettings or $00000018;
   end;


   rc := OpenReader(ReaderDevice, ReaderFirmware, ReaderRc500HWID, ReaderRc500SNR, ReaderSettings, ReaderConsts, ReaderFeatures);

   Screen.Cursor := crDefault;
   btnOK.Enabled := true;
   btnClose.Enabled := true;
   Application.ProcessMessages;

   if rc <> 0 then
   begin

      sbStatus.SimpleText := 'Failed';
      Application.MessageBox(pchar(GetErrorMessage(rc)), 'CSB not found', 0);

   end else
   begin
      sbStatus.SimpleText := 'OK';

      eCsbVersion.Text := ReaderFirmware;
      eCsbChipsetHWID.Text := ReaderRc500HWID;
      eCsbChipsetSNR.Text := ReaderRc500SNR;
      eCsbFeatures.Text := IntToHex(ReaderFeatures, 8);

      lbDeviceInfo.Caption := '';

      case (ReaderSettings and $00000060) of
      $00000000 : lbDeviceInfo.Caption := lbDeviceInfo.Caption + 'RS232';
      $00000020 : lbDeviceInfo.Caption := lbDeviceInfo.Caption + 'RS485';
      $00000040 : lbDeviceInfo.Caption := lbDeviceInfo.Caption + 'USB';
      $00000060 : lbDeviceInfo.Caption := lbDeviceInfo.Caption + 'TCP';
      end;

      lbDeviceInfo.Caption := lbDeviceInfo.Caption + ' (';

      case (ReaderSettings and $00000008) of
      $00000000 : lbDeviceInfo.Caption := lbDeviceInfo.Caption + '38400bps';
      $00000008 : lbDeviceInfo.Caption := lbDeviceInfo.Caption + '115200bps';
      end;

      lbDeviceInfo.Caption := lbDeviceInfo.Caption + '), ';

      case (ReaderSettings and $00000003) of
      $00000000 : lbDeviceInfo.Caption := lbDeviceInfo.Caption + 'OSI 3964';
      $00000001 : lbDeviceInfo.Caption := lbDeviceInfo.Caption + 'ASCII';
      $00000002 : lbDeviceInfo.Caption := lbDeviceInfo.Caption + 'Binary';
      $00000003 : lbDeviceInfo.Caption := lbDeviceInfo.Caption + 'Binary/Bus';
      end;

      if (ReaderSettings and $00000004 = $00000004) then
         lbDeviceInfo.Caption := lbDeviceInfo.Caption + ' (HW ctrl ON)';

      lbDeviceInfo.Caption := 'Connected on ' + ReaderDevice + ' - ' + lbDeviceInfo.Caption;
   end;

   if rc = 0 then
   begin
      ConnectStatus(true);
   end else
   begin
      ConnectStatus(false);
   end;
end;


procedure TfCSBDiag.ckAutoClick(Sender: TObject);
begin
   if gbDevice.Tag <> 0 then exit;
   gbDevice.Tag := 1;
   if Sender = ckAuto then
   begin
      if ckAuto.Checked then
      begin
         ckUsb.Checked := false;
         ckSerial.Checked := false;
         ckOther.Checked := false;
      end else
      begin
         ckUsb.Checked := true;
      end;
   end;
   if Sender = ckUsb then
   begin
      if ckUsb.Checked then
      begin
         ckAuto.Checked := false;
         ckSerial.Checked := false;
         ckOther.Checked := false;
      end else
      begin
         ckAuto.Checked := true;
      end;
   end;
   if Sender = ckSerial then
   begin
      if ckSerial.Checked then
      begin
         ckAuto.Checked := false;
         ckUsb.Checked := false;
         ckOther.Checked := false;
      end else
      begin
         ckAuto.Checked := true;
      end;
   end;
   if Sender = ckOther then
   begin
      if ckOther.Checked then
      begin
         ckAuto.Checked := false;
         ckUsb.Checked := false;
         ckSerial.Checked := false;
      end else
      begin
         ckAuto.Checked := true;
      end;
   end;
   cbSerial.Enabled := ckSerial.Checked;
   eOther.Enabled  := ckOther.Checked;
   gbDevice.Tag := 0;
end;

procedure TfCSBDiag.ApplyDeviceName;
begin
   if ckAuto.Checked then
   begin
      DeviceName := '';
   end else
   if ckUsb.Checked then
   begin
      DeviceName := 'USB';
   end else
   if ckSerial.Checked then
   begin
      DeviceName := cbSerial.ItemsEx[cbSerial.ItemIndex].Caption + ' : ';
      DeviceName := Copy(DeviceName, 1, Pos(' : ', DeviceName) - 1);
   end else
   begin
      DeviceName := eOther.Text;
   end;
end;

procedure TfCSBDiag.btnOKClick(Sender: TObject);
begin
   ApplyDeviceName;
   SaveToReg;
   Close;
end;

procedure TfCSBDiag.btnPollingClick(Sender: TObject);
begin
   if btnPolling.Tag = 0 then
   begin
      btnPolling.Tag := 1;
      btnPolling.Caption := 'Terminate polling';
   end else
   begin
      btnPolling.Tag := 0;
      btnPolling.Caption := 'Contactless polling';
      lbCard.Caption := '';
   end;
end;

procedure TfCSBDiag.btnChangeLevelClick(Sender: TObject);
var
   r : TRegistry;
begin
   r := TRegistry.Create;
   r.RootKey := HKEY_LOCAL_MACHINE;
   r.OpenKey('Software\SpringCard\SpringProx', true);
   r.WriteString('DebugFile', eDebugPath.Text);
   r.WriteString('DebugLevel', spinLevel.Text);
   r.CloseKey;

   r.OpenKey('Software\Pro-active\SpringProx', true);
   r.WriteString('DebugFile', eDebugPath.Text);
   r.WriteString('DebugLevel', spinLevel.Text);
   r.CloseKey;
   r.Free;

   sbStatus.SimpleText := 'Debug File Activated (level=' + spinLevel.Text + ')';
end;

procedure TfCSBDiag.btnCloseClick(Sender: TObject);
begin
   Close;
end;

procedure TfCSBDiag.btnDesactDebugClick(Sender: TObject);
var
   r : TRegistry;
begin
   r := TRegistry.Create;
   r.RootKey := HKEY_LOCAL_MACHINE;
   r.OpenKey('Software\SpringCard\SpringProx', true);
   r.WriteString('DebugFile', eDebugPath.Text);
   r.WriteString('DebugLevel', '0');
   r.CloseKey;

   r.OpenKey('Software\Pro-Active\SpringProx', true);
   r.WriteString('DebugFile', eDebugPath.Text);
   r.WriteString('DebugLevel', '0');
   r.CloseKey;
   r.Free;

   btnDesactDebug.Visible := FALSE;
   btnActDebug.Visible := TRUE;
   btnChangeLevel.Visible := FALSE;
   spinLevel.Visible := FALSE;
   lbDebugLevel.Visible := FALSE;

   spinLevel.Text := '0';

   sbStatus.SimpleText := 'Debug File Desactivated';
end;

procedure TfCSBDiag.btnExplainClick(Sender: TObject);
var
   cpb : dword;
   str : string;
begin
   cpb := StrToIntDef('$' + eCsbFeatures.Text, 0);

   if cpb <> 0 then
   begin
      str := '';
   end else
   begin
      str :=       'Warning : this version of the reader firmware doesn''t provide the GetFeatures function.' + #13 + #10;
      str := str + 'The information displayed here are only default values, and therefore are not reliable.' + #13 + #10;
   end;

                                    str := str + 'Available functions :' + #13 + #10;
   if (cpb and $00000001 <> 0) then str := str + '- ISO 14443 (A and B)' + #13 + #10;
   if (cpb and $00000002 <> 0) then str := str + '- ISO 15693' + #13 + #10;
   if (cpb and $00000020 <> 0) then str := str + '- GemCore smartcard reader' + #13 + #10;
   if (cpb and $00000040 <> 0) then str := str + '- SAGEM MSO CBM fingerprints reader' + #13 + #10;

                                    str := str + 'Supported communication protocols :' + #13 + #10;
                                    str := str + '- OSI 3964' + #13 + #10;
   if (cpb and $00000800 <> 0) then str := str + '- Binary/Bus' + #13 + #10;
   if (cpb and $00000200 <> 0) then str := str + '- Binary' + #13 + #10;
   if (cpb and $00000100 <> 0) then str := str + '- ASCII' + #13 + #10;

   if (cpb and $00001000 <> 0) then str := str + 'Device has an RS-485 physical interface' + #13 + #10;
   if (cpb and $00002000 <> 0) then str := str + 'Device has separate in/out buffers' + #13 + #10;
   if (cpb and $00004000 <> 0) then str := str + 'Device is able to communicate at 115200bps' + #13 + #10;
   if (cpb and $00008000 <> 0) then str := str + 'Device has XXL buffers' + #13 + #10;

   if (cpb and $000F0000 <> 0) then str := str + 'USB device capabilities :' + #13 + #10;
   if (cpb and $00010000 <> 0) then str := str + '- CDC-ACM class ("legacy" virtual comm port)' + #13 + #10;
   if (cpb and $00020000 <> 0) then str := str + '- CCID class (PC/SC smartcard reader)' + #13 + #10;
   if (cpb and $00040000 <> 0) then str := str + '- HID class (keyboard emulation)' + #13 + #10;

   if (cpb and $FFF00000 <> 0) then str := str + 'Advanced capabilities :' + #13 + #10;
   if (cpb and $00400000 <> 0) then str := str + '- FEED (Flash or E2PROM for persistent data storage)' + #13 + #10;
   if (cpb and $01000000 <> 0) then str := str + '- Console access' + #13 + #10;
   if (cpb and $02000000 <> 0) then str := str + '- Noralsy specific functions' + #13 + #10;

   Application.MessageBox(pchar(str), 'Feature list', 0);
end;

procedure TfCSBDiag.btnBenchClick(Sender: TObject);
var
   E128 : dword;
   E256 : dword;
   E768 : dword;
   str  : string;
   rc   : sword;
begin
   Screen.Cursor := crHourglass;
   btnTest.Enabled := false;
   btnOK.Enabled := false;
   btnClose.Enabled := false;
   btnExplain.Enabled := false;
   btnBench.Enabled := false;

   sbStatus.SimpleText := 'Please wait, benchmark in progress (this may take up to 60 seconds)...';
   Application.ProcessMessages;

   rc := BenchReader(E128, E256, E768);

   btnTest.Enabled := true;
   btnOK.Enabled := true;
   btnClose.Enabled := true;
   btnExplain.Enabled := true;
   btnBench.Enabled := true;
   Screen.Cursor := crDefault;

   if rc <> 0 then
   begin

      sbStatus.SimpleText := 'Failed';
      Application.MessageBox(pchar(GetErrorMessage(rc)), 'CSB Benchmark', 0);

   end else
   begin
      sbStatus.SimpleText := 'OK ';   
      str := '';
      
      str := str + 'Echo of 128 bytes : ';
      if E128 = 0 then str := str + 'Test not available'
                  else str := str + 'Passed, ' + IntToStr(E128) + 'ms';
      str := str + #13 + #10;

      str := str + 'Echo of 256 bytes : ';
      if E256 = 0 then str := str + 'Test not available'
                  else str := str + 'Passed, ' + IntToStr(E256) + 'ms';
      str := str + #13 + #10;

      str := str + 'Echo of 768 bytes : ';
      if E768 = 0 then str := str + 'Test not available'
                  else str := str + 'Passed, ' + IntToStr(E768) + 'ms';

      Application.MessageBox(pchar(str), 'Benchmark results', 0);
   end;
end;

procedure TfCSBDiag.btnActDebugClick(Sender: TObject);
var
   r : TRegistry;
begin
   r := TRegistry.Create;
   r.RootKey := HKEY_LOCAL_MACHINE;
   r.OpenKey('Software\SpringCard\SpringProx', true);
   r.WriteString('DebugFile', eDebugPath.Text);
   r.WriteString('DebugLevel', '255');
   r.CloseKey;

   r.OpenKey('Software\Pro-Active\SpringProx', true);
   r.WriteString('DebugFile', eDebugPath.Text);
   r.WriteString('DebugLevel', '255');
   r.CloseKey;
   r.Free;

   btnDesactDebug.Visible := TRUE;
   btnActDebug.Visible := FALSE;
   btnChangeLevel.Visible := TRUE;
   spinLevel.Visible := TRUE;
   lbDebugLevel.Visible := TRUE;

   spinLevel.Text := '255';

   sbStatus.SimpleText := 'Debug File Activated';
end;

procedure TfCSBDiag.btnAdvancedClick(Sender: TObject);
begin
   fCsbCommMore.ShowModal;
end;

procedure TfCSBDiag.lbWebClick(Sender: TObject);
begin
   ShellExecute(0, 'open', 'http://www.springcard.com', nil, nil, 0);
end;

procedure TfCSBDiag.FormClose(Sender: TObject; var Action: TCloseAction);
begin
   UsbNotifier.Free;
   CloseReader;
end;

end.
